@@ -21,11 +21,18 @@
 #define R_DT_val(x)     (lower_tail ? R_D_val(x)  : R_D_Clog(x))
 #define R_DT_Cval(x)    (lower_tail ? R_D_Clog(x) : R_D_val(x))
 
+/* Value of densities at 0 */
+#define R_D_mode(x)                                     \
+    if (x)						\
+	return R_D__0;  /* strictly positive mode */	\
+    else						\
+	return R_PosInf /* mode at 0 */
+
 /*Boundaries*/
 #define R_Q_P01_boundaries(p, _LEFT_, _RIGHT_)          \
     if (log_p) {                                        \
         if(p > 0)                                       \
-            error(_("NAN"));                            \
+            return R_NaN;				\
         if(p == 0) /* upper bound*/                     \
             return lower_tail ? _RIGHT_ : _LEFT_;       \
         if(p == R_NegInf)                               \
@@ -33,7 +40,7 @@
     }                                                   \
     else { /* !log_p */                                 \
         if(p < 0 || p > 1)                              \
-            error(_("NAN"));                            \
+            return R_NaN;				\
         if(p == 0)                                      \
             return lower_tail ? _LEFT_ : _RIGHT_;       \
         if(p == 1)                                      \
