@@ -1,4 +1,4 @@
-/*  ===== actuar: an R package for Actuarial Science =====
+/*  ===== actuar: An R Package for Actuarial Science =====
  *
  *  Functions to compute density, cumulative distribution and quantile
  *  functions, raw and limited moments and to simulate random variates
@@ -36,6 +36,9 @@ double dtrgamma(double x, double shape1, double shape2, double scale,
     if (!R_FINITE(x) || x < 0.0)
         return R_D__0;
 
+    /* handle x == 0 separately */
+    if (x == 0) R_D_mode(shape1 * shape2 > 1);
+
     logu = shape2 * (log(x) - log(scale));
 
     return R_D_exp(log(shape2) + shape1 * logu - exp(logu)
